% MATLAB code for the simulations with N = 50 presented in the paper:
% De Winter, J. C. F., & Dodou, D. (2012). Factor recovery by principal axis factoring and maximum likelihood factor analysis as a function of factor pattern and sample size. Journal of Applied Statistics, 39, 695-710.
% This code was tested with Matlab versions R2011b and R2012b, and re-tested with Matlab R0218b

clear all;close all;clc;warning off all %#ok<CLALL>
reps=2500;% number of repetitions (can be changed)
N=50; % sample size
CASES=1:17; % case numbers to run

[g2_PAF,g2_MLFA,g2_FACT,CC_PAF,CC_MLFA,CC_FACT,MAXCOM_PAF,MAXCOM_MLFA,MAXCOM_FACT]=deal(NaN(reps,max(CASES)));
[AL_PAF,AL_MLFA,AL_FACT]=deal(NaN(reps,max(CASES),36,3));

tol=10^-3;itermax=9999; % tolerance interval and maximum number of iterations for iterative procedures
BLL=.6; % level of all nonzero loadings in baseline pattern matrix
BLp=6; % number of nonzero loadings per factor

for cases=CASES
    disp(cases)
    e=0; % model error
    f=0; % interfactor correlation
    rotmethod=3; % Procrustes rotation
    eps=.15; % parameter determining the distribution of model error
    TL=[repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)]; TL2=TL; % baseline pattern matrix
    nfactors=0;
    switch cases
        case 2  % IFC all faxtors
            f=.7;
        case 3  % model error
            e=.2;eps=.25;
        case 4  %  zero loaders replace
            TL=[repmat([BLL 0 0],BLp-2,1);repmat([0 BLL 0],BLp-2,1);repmat([0 0 BLL],BLp-2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);];
        case 5  % low p
            TL=[repmat([BLL 0 0],BLp/2,1);repmat([0 BLL 0],BLp/2,1);repmat([0 0 BLL],BLp/2,1)];
        case 6 % high p
            TL=[repmat([BLL 0 0],BLp*2,1);repmat([0 BLL 0],BLp*2,1);repmat([0 0 BLL],BLp*2,1)];
        case 7  % unequal loadings of factors
            TL(TL(:,1)>0,1)=BLL+.3;TL(TL(:,3)>0,3)=BLL-.3;
        case 8  % unequal loadings within factors
            temp=find(TL);TL(temp(1:2:end))=BLL+.3;TL(temp(2:2:end))=BLL-.3;
        case 9 % unequal loadings within factors with interfactor correlation
            temp=find(TL);TL(temp(1:2:end))=BLL+.3;TL(temp(2:2:end))=BLL-.3;
            f=.7;
        case 10 % crossloadings
            TL(1,:)=[BLL .6 0 ];
            TL(2,:)=[BLL .6 0 ];
            TL(3,:)=[BLL .6 0 ];
            TL(4,:)=[BLL .6 0 ];
        case 11 % unequal p/f modest
            TL=[repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],3,1)];
        case 12 % unequal p/f severe
            TL=[repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],2,1)];
        case 14 % underextraction -2
            TL=[repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)];
            nfactors=1;
        case 15 % underextraction -1
            TL=[repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)];
            nfactors=2;
        case 16 % overextraction +1
            nfactors=4;
        case 17 % overextraction +2
            nfactors=5;
    end
    
    for REP=1:reps % loop across repetitions
        if cases==13
            TL=TL2-.2+.4*rand(size(TL2)); % random variation of all loadings
        end
        if REP/10==floor(REP/10);disp(REP);end % display repetition counter, displayed each 10 repetitions
        p=size(TL,1); % number of variables
        nfactors2=size(TL,2); % the true number of factors
        if nfactors==0
            nfactors=nfactors2;   % the numbers of factors to extract
        end
        if REP==1
            disp(TL) % display true loading matrix
        end
        
        X=Hong(nfactors2, p,N,TL,e,f,eps);
        r=corrcoef(X);
        
        [L,c1]=paf(r,nfactors,tol,itermax);   % perform PAF, extracting n factors
        L=FArot(L,rotmethod,TL); 
        [L_PAF,TLr1]=ordering(L,TL);
        
        [lding, ~]=factoran(r,nfactors,'delta',tol,'rotate','none','start','Rsquared','xtype','cov'); % perform MLFA, extracting n factors
        c4=sum(lding'.^2,1)';
        L=FArot(lding,rotmethod,TL);
        [L_FACT,TLr4]=ordering(L,TL);
        L_MLFA=L_FACT; TLr3=TLr4; c3=c4;
        
        if max(max(L_PAF))>1.5 || max(max(L_MLFA))>1.5 || max(max(L_FACT))>1.5
            disp('Corrupt solution')
        else
            g2_PAF(REP,cases)=sqrt(mean(mean((L_PAF-TLr1).^2))); % average root mean-squared error of the factor loadings
            g2_MLFA(REP,cases)=sqrt(mean(mean((L_MLFA-TLr3).^2)));
            g2_FACT(REP,cases)=sqrt(mean(mean((L_FACT-TLr4).^2)));
            CC_PAF(REP,cases)=cc(L_PAF,TLr1); % correlation coefficient between factor loadings and true loadings
            CC_MLFA(REP,cases)=cc(L_MLFA,TLr3);
            CC_FACT(REP,cases)=cc(L_FACT,TLr4);
            MAXCOM_PAF(REP,cases)=max(c1);
            MAXCOM_MLFA(REP,cases)=max(c3);
            MAXCOM_FACT(REP,cases)=max(c4);
            AL_PAF(REP,cases,1:p,1:size(L_PAF,2))=L_PAF-TLr1; % g per loading coefficient
            AL_MLFA(REP,cases,1:p,1:size(L_MLFA,2))=L_MLFA-TLr3;
            AL_FACT(REP,cases,1:p,1:size(L_FACT,2))=L_FACT-TLr4;
        end
    end
end
%%
close all
screen1=find(MAXCOM_PAF>=.998); % Heywood cases
screen3=find(MAXCOM_MLFA>=.998);
[screen1a,screen1b]=find(MAXCOM_PAF>=.998);
[screen3a,screen3b]=find(MAXCOM_MLFA>=.998);

disp('Table 2. HW PAF and HW MLFA:')
disp(100*[sum(MAXCOM_PAF>=.998)'  sum(MAXCOM_MLFA>=.998)']./[sum(MAXCOM_PAF>=.0001)' sum(MAXCOM_MLFA>=.0001)'])

g2_PAF_screened=g2_PAF;g2_PAF_screened(screen1)=NaN;g2_PAF_screened(screen3)=NaN;
g2_MLFA_screened=g2_MLFA;g2_MLFA_screened(screen1)=NaN;g2_MLFA_screened(screen3)=NaN;

temp4=double(g2_PAF_screened>g2_MLFA_screened); % proportion of MLFA outperforming solutions
temp4(isnan(g2_PAF_screened+g2_MLFA_screened))=NaN;

CD=NaN(1,size(g2_MLFA,2));
for i=1:size(g2_MLFA,2)
    CD(i)=cohend(g2_MLFA_screened(:,i),g2_PAF_screened(:,i)); % Cohens d between the g values of PAF and MLFA
end

disp('Table 2. g PAF, g MLFA, g PAF>MLFA, and d:')
disp([round(10000*nanmean(g2_PAF_screened)')/10000 round(10000*nanmean(g2_MLFA_screened)')/10000 round(1000*nanmean(temp4)')/1000 round(1000*CD')/1000])

AL_PAF2=AL_PAF;
AL_MLFA2=AL_MLFA;

disp('g per loading coefficient for each case (Cases 11 & 9 shown in Table 3):')
for V=1:size(AL_PAF,2)
    disp(['Case ' num2str(V)])
    AL_PAF2(screen1a(screen1b==V),V,:,:)=NaN;
    AL_PAF2(screen3a(screen3b==V),V,:,:)=NaN;
    AL_MLFA2(screen1a(screen1b==V),V,:,:)=NaN;
    AL_MLFA2(screen3a(screen3b==V),V,:,:)=NaN;
    disp([squeeze(nanmean(AL_PAF2(:,V,:,:).^2)).^.5 squeeze(nanmean(AL_MLFA2(:,V,:,:).^2)).^.5])
end
%%
SP=NaN(25,18);
for i=1:size(SP,1)
    TL=[repmat([.6 0 0],6,1);repmat([0 .6 0],6,1);repmat([0 0 .6],6,1)];
    [~,F]=Hong(3,size(SP,2),50,TL,.2,0,.25);
    SP(i,:)=eig(F*F'); % eigenvalue
end

TL=[repmat([.6 0 0],6,1);repmat([0 .6 0],6,1);repmat([0 0 .6],6,1)];
[~,F]=Hong(3,size(SP,2),50,TL,0,0,0);
SP_clean=eig(F*F');

figure;hold on
plot(SP(:,end:-1:1)','k-s','color',[.5 .5 .5],'MarkerSize',5,'MarkerFaceColor', [.5 .5 .5])
plot(SP_clean(end:-1:1)','k-o','LineWidth',3,'MarkerSize',15,'MarkerFaceColor', 'w')
xlabel('Factor number', 'FontSize',20)
ylabel('Eigenvalue', 'FontSize',20)
h=findobj(gcf,'FontName','Helvetica');
set(h,'FontSize',20)
set(gca,'Xlim',[.8 0.2+size(SP,2)],'XTick',1:size(SP,2))

figure;hold on
plot(g2_PAF_screened(:,9),g2_MLFA_screened(:,9),'ko')
plot([0 .65],[0 .65],'r','LineWidth',2)
xlabel('\itg\rm PAF', 'FontSize',20)
ylabel('\itg\rm MLFA', 'FontSize',20)
h=findobj(gcf,'FontName','Helvetica');
set(h,'FontSize',20)
set(gca,'Xlim',[0 .65],'Ylim',[0 .65])