% MATLAB code for the simulations with  eight different sample sizes between 25 and 5000 presented in the paper:
% De Winter, J. C. F., & Dodou, D. (2012). Factor recovery by principal axis factoring and maximum likelihood factor analysis as a function of factor pattern and sample size. Journal of Applied Statistics, 39, 695-710.
% This code was tested with Matlab versions R2011b and R2012b, and re-tested with Matlab R0218b

clear all;close all;clc;warning off all %#ok<CLALL>
reps=2500;% number of repetitions (can be changed)
CASES=[9 11 12 14 17]; % case numbers to run
SAMPLESIZES=[25 50 100 250 500 1000 2500 5000]; % sample sizes

[g2_PAF,g2_MLFA,K_PAF,K_MLFA,MAXCOM_PAF,MAXCOM_MLFA]= deal(NaN(reps,length(SAMPLESIZES),max(CASES)));

tol=10^-3;itermax=9999; % tolerance interval and maximum number of iterations for iterative procedures
BLL=.6; % level of all nonzero loadings in baseline pattern matrix
BLp=6; % number of nonzero loadings per factor
for cases=CASES
    disp(cases)
    for samplesizes=1:length(SAMPLESIZES)
        N=SAMPLESIZES(samplesizes);
        disp(cases)
        e=0; % model error
        f=0; % interfactor correlation
        rotmethod=3; % Procrustes rotation
        eps=.15; % parameter determining the distribution of model error
        TL= [repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)]; TL2=TL; % baseline pattern matrix
        nfactors=0;
        switch cases
            case 2  % IFC all faxtors
                f=.7;
            case 3  % model error
                e=.2;eps=.25;
            case 4  %  zero loaders replace
                TL= [repmat([BLL 0 0],BLp-2,1);repmat([0 BLL 0],BLp-2,1);repmat([0 0 BLL],BLp-2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);];
            case 5  % low p
                TL= [repmat([BLL 0 0],BLp/2,1);repmat([0 BLL 0],BLp/2,1);repmat([0 0 BLL],BLp/2,1)];
            case 6 % high p
                TL= [repmat([BLL 0 0],BLp*2,1);repmat([0 BLL 0],BLp*2,1);repmat([0 0 BLL],BLp*2,1)];
            case 7  % unequal loadings of factors
                TL(TL(:,1)>0,1)=BLL+.3;TL(TL(:,3)>0,3)=BLL-.3;
            case 8  % unequal loadings within factors
                temp=find(TL);TL(temp(1:2:end))=BLL+.3;TL(temp(2:2:end))=BLL-.3;
            case 9 % unequal loadings within factors with interfactor correlation
                temp=find(TL);TL(temp(1:2:end))=BLL+.3;TL(temp(2:2:end))=BLL-.3;
                f=.7;
            case 10 % crossloadings
                TL(1,:)=[BLL .6 0];
                TL(2,:)=[BLL .6 0];
                TL(3,:)=[BLL .6 0];
                TL(4,:)=[BLL .6 0];
            case 11 % unequal p/f modest
                TL= [repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],3,1)];
            case 12 % unequal p/f severe
                TL= [repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],2,1)];
            case 14 % underextraction -2
                TL= [repmat([BLL 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)];
                nfactors=1;
            case 15 % underextraction -1
                TL= [repmat([BLL+0.01 0 0],BLp,1);repmat([0 BLL 0],BLp,1);repmat([0 0 BLL],BLp,1)];
                nfactors=2;
            case 16 % overextraction +1
                nfactors=4;
            case 17 % overextraction +2
                nfactors=5;
        end
        
        for REP=1:reps % loop across repetitions
            if cases==13
                TL= TL2-.2+.4*rand(size(TL2)); % random variation of all loadings
            end
            if REP/10==floor(REP/10);disp(REP);end % display repetition counter, displayed each 10 repetitions
            p=size(TL,1); % number of variables
            nfactors2=size(TL,2); % the true number of factors
            if nfactors==0
                nfactors=nfactors2;   % the numbers of factors to extract
            end
            if REP==1
                disp(TL) % display true loading matrix
            end
            
            X=Hong(nfactors2, p,N,TL,e,f,eps);
            r=corrcoef(X);
            
            [L,c1] =paf(r,nfactors,tol,itermax); % perform PAF, extracting m factors
            L=FArot(L,rotmethod,TL);
            [L_PAF,TLr1] = ordering(L,TL);
            
            [lding, ~]=factoran(r,nfactors,'delta',tol,'rotate','none','start','Rsquared','xtype','cov'); % perform MLFA, extracting n factors
            c4=sum(lding'.^2,1)';
            L=FArot(lding,rotmethod,TL);
            [L_FACT,TLr4]=ordering(L,TL);
            L_MLFA=L_FACT; TLr3=TLr4;c3=c4;
            
            if max(max(L_PAF))>1.5 || max(max(L_MLFA))>1.5 || max(max(L_FACT))>1.5
                disp('Corrupt solution')
            else
                g2_PAF(REP,samplesizes,cases)=sqrt(mean(mean((L_PAF-TLr1).^2))); % average root mean-squared error of the factor loadings
                g2_MLFA(REP,samplesizes,cases)=sqrt(mean(mean((L_MLFA-TLr3).^2)));
                K_PAF(REP,samplesizes,cases)=cc(L_PAF,TLr1); % correlation coefficient between factor loadings and true loadings
                K_MLFA(REP,samplesizes,cases)=cc(L_MLFA,TLr3);
                MAXCOM_PAF(REP,samplesizes,cases)=max(c1);
                MAXCOM_MLFA(REP,samplesizes,cases)=max(c3);
            end
        end
    end
end
%%
close all
screen1=find(MAXCOM_PAF>=.998); % Heywood cases
screen3=find(MAXCOM_MLFA>=.998);

g2_PAF_screened=g2_PAF;g2_PAF_screened(screen1)=NaN;g2_PAF_screened(screen3)=NaN;
g2_MLFA_screened=g2_MLFA;g2_MLFA_screened(screen1)=NaN;g2_MLFA_screened(screen3)=NaN;

temp4=double(g2_PAF_screened>g2_MLFA_screened);
temp4(isnan(g2_PAF_screened+g2_MLFA_screened))=NaN;

CD=NaN(length(SAMPLESIZES),max(CASES));
for j=CASES
    for i=1:size(g2_MLFA,2)
        CD(i,j)=cohend(g2_MLFA_screened(:,i,j),g2_PAF_screened(:,i,j));
    end
end
%%
for cases=CASES
    figure;
    loglog(SAMPLESIZES,nanmean(g2_PAF_screened(:,:,cases)),'k-s','MarkerSize',16,'MarkerFaceColor',[1 1 1])
    hold on
    loglog(SAMPLESIZES,nanmean(g2_MLFA_screened(:,:,cases)),'k-o','MarkerSize',16,'MarkerFaceColor',[0 0 0]);
    legend('PAF','MLFA')
    xlabel('Sample size', 'FontSize',20)
    ylabel('\itg', 'FontSize',20)
    h = findobj(gcf,'FontName','Helvetica');
    set(h,'FontSize',20)
    set(gca,'Ylim',[.01 .4],'Xlim',[min(SAMPLESIZES) max(SAMPLESIZES)])
    
    figure;
    semilogx(SAMPLESIZES,CD(:,cases),'k-s','MarkerSize',16,'MarkerFaceColor','k')
    hold on
    semilogx(SAMPLESIZES,nanmean(temp4(:,:,cases)),'k-o','MarkerSize',16,'MarkerFaceColor','w')
    semilogx(SAMPLESIZES,sum(MAXCOM_PAF(:,:,cases)>=.998)'./sum(MAXCOM_PAF(:,:,cases)>=.0001)','k-<','MarkerSize',16,'MarkerFaceColor',[.5 .5 .5])
    semilogx(SAMPLESIZES,sum(MAXCOM_MLFA(:,:,cases)>=.998)'./sum(MAXCOM_MLFA(:,:,cases)>=.0001)','k-v','MarkerSize',16,'MarkerFaceColor',[.5 .5 .5])
    legend('\itd','PAF > MLFA','Heywood PAF','Heywood MLFA')
    xlabel('Sample size', 'FontSize',20)
    ylabel('Proportion / \itd')
    h = findobj(gcf,'FontName','Helvetica');
    set(h,'FontSize',20)
    set(gca,'Xlim',[min(SAMPLESIZES) max(SAMPLESIZES)])
end