% MATLAB code for the empirical (DBQ) study presented in the paper:
% De Winter, J. C. F., & Dodou, D. (2012). Factor recovery by principal axis factoring and max  imum likelihood factor analysis as a function of factor pattern and sample size. Journal of Applied Statistics, 39, 695-710.
% This code was tested with MATLAB versions R2011b and R2012b, and re-tested with MATLAB R0218b

close all;clear all;clc %#ok<CLALL>
for ii=1:3 % 1=two factors; 2=weak factor; 3=one factor (underextraction)
    clearvars -except ii;
    load DBQ.mat
    X=DBQ;
    
    if ii==1||ii==3
        V=[28 36 18 25 20 7 9 17 4 30 8 32]; % 6 items with the highest loadings per factor
    else
        V=[23 3 9 17 4 30 8]; % 5 strong for factor 1, 2 weak for factor 2
    end
    X=X(:,V);
    r=corrcoef(X);
    
    LB=factoran(r,2,'rotate','promax','rotate','none','start','Rsquared','xtype','cov');
    
    L1=factoran(r,2,'rotate','none','rotate','none','start','Rsquared','xtype','cov'); % perform MLFA, extracting 2 factors
    L1=FArot(L1,1);L1=ordering(L1,LB);
    
    L2=paf(r,2,10^-3,9999); % perform PAF, extracting 2 factors
    L2=FArot(L2,1);L2=ordering(L2,LB);
    
    LB=0.5*(L1+L2); % average of PAF and MLFA solutions (Table 4) as reference matrix
    
    if ii==1
        disp('Table 4')
        disp([V' LB])
    end
    if ii==2
        disp('Table 5')
        disp([V' LB])
    end
    if ii==3
        L1_1=factoran(r,1,'rotate','none','rotate','none','start','Rsquared','xtype','cov'); % perform MLFA, extracting 1 factor
        L2_1=paf(r,1,10^-3,9999); % perform PAF, extracting 1 factor
        disp('Table 6')
        disp([V' L2_1 L1_1 ])
    end
    
    %% Sampling study with eight sample sizes
    X2=X; LL=size(X2,1);
    NN=[25 50 100 250 500 1000 2500 5000]; %
    REPS=2500; % number of repetitions
    if ii==1||ii==2
        nfactors=2; % number of factors to be extracted
    else
        nfactors=1;% number of factors to be extracted
    end
    [CC,LK,g2,HW]=deal(NaN(length(NN),REPS,2));
    for i2=1:length(NN)
        N=NN(i2);
        for counter=1:REPS
            disp(counter)
            SN=ceil(rand(ceil(N*3),1)*LL); SN=unique(SN);temp=randperm(length(SN));temp=temp(1:N); SN=SN(temp); % quick way to draw a random sample with replacement
            clear l1
            X3=zscore(X2(SN,:));r=corrcoef(X3);
            try
                [l1,cpaf]=paf(r,nfactors,10^-3,9999); % perform PAF, extracting n factors
                if ii==1||ii==2
                    l1=FArot(l1,3,LB);
                end
                [l1,LB1]=ordering(l1,LB);
                
                l2=factoran(r,nfactors,'delta',10^-3,'rotate','none','start','Rsquared','xtype','cov'); % perform MLFA, extracting n factors
                cmlfa=sum(l2'.^2,1)';
                if ii==1||ii==2
                    l2=FArot(l2,3,LB);
                end
                [l2,LB2]=ordering(l2,LB);
                
                if max(max(l2))>1.5 || max(max(l1))>1.5
                  %  disp('No FA could be conducted')
                else
                    g2(i2,counter,1)=sqrt(mean(mean((l1-LB1).^2))); % average root mean-squared error of the factor loadings
                    HW(i2,counter,1)=max(cpaf>=.998); % Heywood cases
                    g2(i2,counter,2)=sqrt(mean(mean((l2-LB2).^2)));
                    HW(i2,counter,2)=max(cmlfa>=.998);
                end
                if HW(i2,counter,1)==1 || HW(i2,counter,2)==1
                    g2(i2,counter,1)=NaN;
                    g2(i2,counter,2)=NaN;
                end
            catch error
            end
        end
    end
    %%
    figure;
    loglog(NN,squeeze(nanmean(g2(:,:,1),2)),'k-s','MarkerSize',16,'MarkerFaceColor',[1 1 1]);hold on;
    loglog(NN,squeeze(nanmean(g2(:,:,2),2)),'k-o','MarkerSize',16,'MarkerFaceColor',[0 0 0]);
    legend('PAF','MLFA')
    xlabel('Sample size', 'FontSize',20);ylabel('\itg', 'FontSize',20)
    h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
    set(gca,'Ylim',[.01 .4],'Xlim',[min(NN) max(NN)])
    
    CD=NaN(1,size(g2,1));
    for i=1:size(g2,1);CD(i)=cohend(squeeze(g2(i,:,2)),squeeze(g2(i,:,1)));end
    
    temp=double(squeeze(g2(:,:,1))>squeeze(g2(:,:,2)));
    temp(isnan(squeeze(g2(:,:,1))+squeeze(g2(:,:,2))))=NaN;
    
    figure
    semilogx(NN,CD,'k-s','MarkerSize',16,'MarkerFaceColor','k');hold on
    semilogx(NN,nanmean(temp,2)','k-o','MarkerSize',16,'MarkerFaceColor','w')
    semilogx(NN,nanmean(squeeze(HW(:,:,1)),2)','k-<','MarkerSize',16,'MarkerFaceColor','w')
    semilogx(NN,nanmean(squeeze(HW(:,:,2)),2)','k-v','MarkerSize',16,'MarkerFaceColor','w')
    legend('\itd','PAF > MLFA','Heywood PAF','Heywood MLFA','location','northoutside','orientation','horizontal')
    xlabel('Sample size', 'FontSize',20)
    h = findobj(gcf,'FontName','Helvetica');set(h,'FontSize',20)
    set(gca,'Xlim',[min(NN) max(NN)],'ylim',[-2 1])
end